import json
from   operator                 import attrgetter
from   pathlib                  import Path
import pytest
from   wheel_inspect.classes    import WheelFile
from   wheel_inspect.errors     import WheelValidationError
from   wheel_inspect.inspecting import verify_record

@pytest.mark.parametrize('whlfile', [
    p for p in (Path(__file__).with_name('data') / 'bad-wheels').iterdir()
      if p.suffix == '.whl'
], ids=attrgetter("name"))
def test_verify_bad_wheels(whlfile):
    with whlfile.with_suffix('.json').open() as fp:
        expected = json.load(fp)
    with WheelFile(whlfile) as whl:
        with pytest.raises(WheelValidationError) as excinfo:
            verify_record(whl, whl.get_record())
        assert type(excinfo.value).__name__ == expected["type"]
        assert str(excinfo.value) == expected["str"]



import json
from   operator      import attrgetter
from   pathlib       import Path
from   jsonschema    import validate
import pytest
from   wheel_inspect import DIST_INFO_SCHEMA, WHEEL_SCHEMA, \
                                inspect_dist_info_dir, inspect_wheel

@pytest.mark.parametrize('whlfile', [
    p for p in (Path(__file__).with_name('data') / 'wheels').iterdir()
      if p.suffix == '.whl'
], ids=attrgetter("name"))
def test_inspect_wheel(whlfile):
    with whlfile.with_suffix('.json').open() as fp:
        expected = json.load(fp)
    inspection = inspect_wheel(whlfile)
    assert inspection == expected
    validate(inspection, WHEEL_SCHEMA)

@pytest.mark.parametrize('didir', [
    p for p in (Path(__file__).with_name('data') / 'dist-infos').iterdir()
      if p.is_dir()
], ids=attrgetter("name"))
def test_inspect_dist_info_dir(didir):
    with open(str(didir) + '.json') as fp:
        expected = json.load(fp)
    inspection = inspect_dist_info_dir(didir)
    assert inspection == expected
    validate(inspection, DIST_INFO_SCHEMA)



import base64
from   binascii    import hexlify, unhexlify
from   collections import OrderedDict
import csv
import hashlib
import re
import attr
from   .           import errors

@attr.s
class Record:
    files = attr.ib()

    def __iter__(self):
        return iter(self.files.values())

    def __contains__(self, filename):
        return filename in self.files

    def for_json(self):
        return [e.for_json() for e in self.files.values()]


@attr.s
class RecordEntry:
    path             = attr.ib()
    digest_algorithm = attr.ib()
    #: The digest in hex format
    digest           = attr.ib()
    size             = attr.ib()

    @classmethod
    def from_csv_fields(cls, fields):
        try:
            path, alg_digest, size = fields
        except ValueError:
            raise errors.RecordLengthError(
                fields[0] if fields else None,
                len(fields),
            )
        if not path:
            raise errors.EmptyPathError()
        elif '//' in path or '.' in path.split('/') or '..' in path.split('/'):
            raise errors.NonNormalizedPathError(path)
        elif path.startswith('/'):
            raise errors.AbsolutePathError(path)
        if alg_digest:
            digest_algorithm, digest = alg_digest.split('=', 1)
            if digest_algorithm not in hashlib.algorithms_guaranteed:
                raise errors.UnknownDigestError(path, digest_algorithm)
            elif digest_algorithm in ('md5', 'sha1'):
                raise errors.WeakDigestError(path, digest_algorithm)
            sz = (getattr(hashlib, digest_algorithm)().digest_size * 8 + 5) // 6
            if not re.fullmatch(r'[-_0-9A-Za-z]{%d}' % (sz,), digest):
                raise errors.MalformedDigestError(path,digest_algorithm,digest)
            digest = record_digest2hex(digest)
        else:
            digest_algorithm, digest = None, None
        if size:
            try:
                size = int(size)
            except ValueError:
                raise errors.MalformedSizeError(path, size)
        else:
            size = None
        if digest is None and size is not None:
            raise errors.EmptyDigestError(path)
        elif digest is not None and size is None:
            raise errors.EmptySizeError(path)
        return cls(
            path = path,
            digest_algorithm = digest_algorithm,
            digest = digest,
            size = size,
        )

    def for_json(self):
        return {
            "path": self.path,
            "digests": {self.digest_algorithm: hex2record_digest(self.digest)}
                        if self.digest is not None else {},
            "size": self.size,
        }


def parse_record(fp):
    # Format defined in PEP 376
    files = OrderedDict()
    for fields in csv.reader(fp, delimiter=',', quotechar='"'):
        if not fields:
            continue
        entry = RecordEntry.from_csv_fields(fields)
        if entry.path in files and files[entry.path] != entry:
            raise errors.RecordConflictError(entry.path)
        files[entry.path] = entry
    return Record(files)

def hex2record_digest(data):
    return base64.urlsafe_b64encode(unhexlify(data)).decode('us-ascii')\
                                                    .rstrip('=')

def record_digest2hex(data):
    pad = '=' * (4 - (len(data) & 3))
    return hexlify(base64.urlsafe_b64decode(data + pad)).decode('us-ascii')



from copy import deepcopy

#: A `JSON Schema <http://json-schema.org>`_ for the structure returned by
#: `inspect_dist_info_dir()` and by `inspect()` when called on a `DistInfoDir`.
#: It is the same as `WHEEL_SCHEMA`, but without the ``"filename"``,
#: ``"project"``, ``"version"``, ``"buildver"``, ``"pyver"``, ``"abi"``,
#: ``"arch"``, and ``"file"`` keys.
DIST_INFO_SCHEMA = {
    "$schema": "http://json-schema.org/draft-07/schema#",
    "type": "object",
    "required": [
        "valid",
        "dist_info",
        "derived"
    ],
    "additionalProperties": False,
    "properties": {
        "valid": {
            "type": "boolean",
            "description": "Whether the wheel is well-formed with an accurate RECORD"
        },

        "validation_error": {
            "type": "object",
            "description": "If the wheel is invalid, this field contains information on the `WheelValidationError` raised.",
            "required": ["type", "str"],
            "additionalProperties": False,
            "properties": {
                "type": {
                    "type": "string",
                    "description": "The name of the type of exception raised"
                },
                "str": {
                    "type": "string",
                    "description": "The exception's error message"
                }
            }
        },

        "dist_info": {
            "type": "object",
            "description": "JSONifications of files from the wheel's .dist-info directory",
            "additionalProperties": False,
            "properties": {

                "metadata": {
                    "type": "object",
                    "additionalProperties": {
                        "type": "array",
                        "items": {"type": "string"}
                    },
                    "properties": {
                        "metadata_version": {"type": "string"},
                        "name": {"type": "string"},
                        "version": {"type": "string"},
                        "summary": {"type": ["null", "string"]},
                        "description": {
                            "oneOf": [
                                {"type": "null"},
                                {
                                    "type": "object",
                                    "requires": ["length"],
                                    "additionalProperties": False,
                                    "properties": {
                                        "length": {"type": "integer"}
                                    }
                                }
                            ]
                        },
                        "requires_dist": {
                            "type": "array",
                            "items": {
                                "type": "object",
                                "required": [
                                    "name",
                                    "url",
                                    "extras",
                                    "specifier",
                                    "marker"
                                ],
                                "additionalProperties": False,
                                "properties": {
                                    "name": {"type": "string"},
                                    "url": {"type": ["null", "string"]},
                                    "extras": {
                                        "type": "array",
                                        "items": {"type": "string"}
                                    },
                                    "specifier": {"type": "string"},
                                    "marker": {"type": ["null", "string"]}
                                }
                            }
                        },
                        "project_url": {
                            "type": "array",
                            "items": {
                                "type": "object",
                                "requires": ["label", "url"],
                                "additionalProperties": False,
                                "properties": {
                                    "label": {"type": ["null", "string"]},
                                    "url": {"type": "string"}
                                }
                            }
                        },
                        "requires_python": {"type": "string"},
                        "author": {"type": ["null", "string"]},
                        "author_email": {"type": ["null", "string"]},
                        "download_url": {"type": ["null", "string"]},
                        "home_page": {"type": ["null", "string"]},
                        "license": {"type": ["null", "string"]},
                        "maintainer": {"type": ["null", "string"]},
                        "maintainer_email": {"type": ["null", "string"]},
                        "keywords": {"type": ["null", "string"]},
                        "description_content_type": {"type": ["null", "string"]}
                    }
                },

                "record": {
                    "type": "array",
                    "items": {
                        "type": "object",
                        "required": ["path", "digests", "size"],
                        "additionalProperties": False,
                        "properties": {
                            "path": {"type": "string"},
                            "digests": {
                                "type": "object",
                                "properties": {
                                    "md5": {"type": "string", "pattern": "^[-_0-9A-Za-z]{22}$"},
                                    "sha1": {"type": "string", "pattern": "^[-_0-9A-Za-z]{27}$"},
                                    "sha256": {"type": "string", "pattern": "^[-_0-9A-Za-z]{43}$"},
                                    "sha512": {"type": "string", "pattern": "^[-_0-9A-Za-z]{86}$"}
                                }
                            },
                            "size": {"type": ["null", "integer"]}
                        }
                    }
                },

                "wheel": {
                    "type": "object",
                    "required": ["wheel_version", "generator", "root_is_purelib", "tag"],
                    "additionalProperties": {
                        "type": "array",
                        "items": {"type": "string"}
                    },
                    "properties": {
                        "wheel_version": {"type": "string"},
                        "generator": {"type": "string"},
                        "root_is_purelib": {"type": "boolean"},
                        "tag": {
                            "type": "array",
                            "items": {"type": "string"}
                        },
                        "build": {"type": "string"},
                        "BODY": {"type": "string"}
                    }
                },

                "dependency_links": {
                    "type": "array",
                    "items": {"type": "string"}
                },

                "entry_points": {
                    "type": "object",
                    "additionalProperties": {
                        "type": "object",
                        "additionalProperties": {
                            "type": "object",
                            "required": ["module", "attr", "extras"],
                            "additionalProperties": False,
                            "properties": {
                                "module": {"type": "string"},
                                "attr": {"type": ["null", "string"]},
                                "extras": {
                                    "type": "array",
                                    "items": {"type": "string"}
                                }
                            }
                        }
                    }
                },

                "namespace_packages": {
                    "type": "array",
                    "items": {"type": "string"}
                },

                "top_level": {
                    "type": "array",
                    "items": {"type": "string"}
                },

                "zip_safe": {"type": "boolean"}
            }
        },

        "derived": {
            "type": "object",
            "description": "Information derived from `dist_info`",
            "required": [
                "readme_renders",
                "description_in_body",
                "description_in_headers",
                "keywords",
                "keyword_separator",
                "dependencies",
                "modules"
            ],
            "additionalProperties": False,
            "properties": {
                "readme_renders": {
                    "type": ["null", "boolean"],
                    "description": "Whether the description's markup can be rendered successfully on PyPI.  A value of `null` indicates that there is no description."
                },
                "description_in_body": {
                    "type": "boolean",
                    "description": "Whether the description is present as the message body in the METADATA file"
                },
                "description_in_headers": {
                    "type": "boolean",
                    "description": "Whether the description is present as a header field in the METADATA file"
                },
                "keywords": {
                    "type": "array",
                    "items": {"type": "string"},
                    "uniqueItems": True,
                    "description": "The wheel's keywords string, split on what appears to be the appropriate separator"
                },
                "keyword_separator": {
                    "enum": [" ", ",", None],
                    "description": "The apparent appropriate separator for the wheel's keywords string.  A value of `null` indicates that the keywords string is undefined."
                },
                "dependencies": {
                    "type": "array",
                    "items": {"type": "string"},
                    "uniqueItems": True,
                    "description": "The names of all of the projects listed in the wheel's Requires-Dist"
                },
                "modules": {
                    "type": "array",
                    "items": {"type": "string"},
                    "uniqueItems": True,
                    "description": "A list of Python modules installed by the wheel"
                }
            }
        }
    }
}

#: A `JSON Schema <http://json-schema.org>`_ for the structure returned by
#: `inspect_wheel()` and by `inspect()` when called on a `WheelFile`.
WHEEL_SCHEMA = deepcopy(DIST_INFO_SCHEMA)

WHEEL_SCHEMA["required"].extend([
    "filename",
    "project",
    "version",
    "buildver",
    "pyver",
    "abi",
    "arch",
    "file",
])

WHEEL_SCHEMA["properties"].update({
    "filename": {
        "type": "string",
        "description": "The filename of the wheel"
    },
    "project": {
        "type": "string",
        "description": "The name of the wheel's project as extracted from the filename"
    },
    "version": {
        "type": "string",
        "description": "The wheel's project version as extracted from the filename"
    },
    "buildver": {
        "type": ["string", "null"],
        "description": "The wheel's build tag as extracted from the filename; `null` if there is no build tag"
    },
    "pyver": {
        "type": "array",
        "items": {"type": "string"},
        "description": "A list of Python versions with which the wheel is compatible as extracted from the filename"
    },
    "abi": {
        "type": "array",
        "items": {"type": "string"},
        "description": "A list of ABIs with which the wheel is compatible as extracted from the filename"
    },
    "arch": {
        "type": "array",
        "items": {"type": "string"},
        "description": "A list of architectures with which the wheel is compatible as extracted from the filename"
    },
    "file": {
        "type": "object",
        "required": ["size", "digests"],
        "additionalProperties": False,
        "properties": {
            "size": {"type": "integer"},
            "digests": {
                "type": "object",
                "required": ["md5", "sha256"],
                "additionalProperties": False,
                "properties": {
                    "md5": {"type": "string", "pattern": "^[0-9A-Fa-f]{32}$"},
                    "sha256": {"type": "string", "pattern": "^[0-9A-Fa-f]{64}$"}
                }
            }
        }
    },
})

#: Alias for `WHEEL_SCHEMA`.  Deprecated; use `WHEEL_SCHEMA` instead.
SCHEMA = WHEEL_SCHEMA



from   email.message   import EmailMessage
import hashlib
from   keyword         import iskeyword
import re
from   typing          import List, Optional, Tuple
from   packaging.utils import canonicalize_name, canonicalize_version
from   .errors         import DistInfoError

DIGEST_CHUNK_SIZE = 65535

DIST_INFO_DIR_RGX = re.compile(
    r'[A-Za-z0-9](?:[A-Za-z0-9._]*[A-Za-z0-9])?-[A-Za-z0-9_.!+]+\.dist-info'
)

DATA_DIR_RGX = re.compile(
    r'[A-Za-z0-9](?:[A-Za-z0-9._]*[A-Za-z0-9])?-[A-Za-z0-9_.!+]+\.data'
)

# <https://discuss.python.org/t/identifying-parsing-binary-extension-filenames/>
MODULE_EXT_RGX = re.compile(
    r'(?<=.)\.(?:py|pyd|so|[-A-Za-z0-9_]+\.(?:pyd|so))\Z'
)

def extract_modules(filelist):
    modules = set()
    for fname in filelist:
        parts = fname.split('/')
        if not parts:
            continue
        if len(parts) > 2 and is_data_dir(parts[0]) \
                and parts[1] in ('purelib', 'platlib'):
            parts = parts[2:]
        m = MODULE_EXT_RGX.search(parts[-1])
        if m is None:
            continue
        parts[-1] = parts[-1][:m.start()]
        if not all(p.isidentifier() and not iskeyword(p) for p in parts):
            continue
        if parts[-1] == '__init__' and len(parts) > 1:
            parts.pop()
        modules.add('.'.join(parts))
    return sorted(modules)

def split_keywords(kwstr):
    # cf. `format_tags()` in Warehouse <https://git.io/fA1AT>, which seems to
    # be the part of PyPI responsible for splitting keywords up for display

    # cf. how wheel handles keywords:
    #keywords = re.split(r'[\0-,]+', kwstr)

    # Based on how pydigger.com seems to handle keywords (See
    # <https://pydigger.com/keywords>):
    if ',' in kwstr:
        return ([k for k in map(str.strip, kwstr.split(',')) if k], ',')
    else:
        return (kwstr.split(), ' ')

def strfield(s):
    return None if s is None or s.strip() in ('', 'UNKNOWN') else s

def fieldnorm(s):
    return s.lower().replace('-', '_')

def unique_projects(projects):
    seen = set()
    for p in projects:
        pn = canonicalize_name(p)
        if pn not in seen:
            yield p
        seen.add(pn)

def digest_file(fp, algorithms):
    digests = {alg: getattr(hashlib, alg)() for alg in algorithms}
    for chunk in iter(lambda: fp.read(DIGEST_CHUNK_SIZE), b''):
        for d in digests.values():
            d.update(chunk)
    return {k: v.hexdigest() for k,v in digests.items()}

def split_content_type(s):
    msg = EmailMessage()
    msg["Content-Type"] = s
    ct = msg["Content-Type"]
    return (ct.maintype, ct.subtype, ct.params)

def is_dist_info_dir(name):
    return DIST_INFO_DIR_RGX.fullmatch(name) is not None

def is_data_dir(name):
    return DATA_DIR_RGX.fullmatch(name) is not None

def is_dist_info_path(path, name):
    pre, _, post = path.partition('/')
    return is_dist_info_dir(pre) and post == name

def yield_lines(fp):
    # Like pkg_resources.yield_lines(fp), but without the dependency on
    # pkg_resources
    for line in fp:
        line = line.strip()
        if line and not line.startswith('#'):
            yield line

def find_dist_info_dir(namelist: List[str], project: str, version: str) \
        -> Tuple[str, Optional[str]]:
    """
    Given a list ``namelist`` of files in a wheel for a project ``project`` and
    version ``version``, find & return the name of the wheel's ``.dist-info``
    directory.

    :raises DistInfoError: if there is no unique ``.dist-info`` directory in
        the input
    :raises DistInfoError: if the name & version of the ``.dist-info``
        directory are not normalization-equivalent to ``project`` & ``version``
    """
    canon_project = canonicalize_name(project)
    canon_version = canonicalize_version(version.replace('_', '-'))
    dist_info_dirs = set()
    for n in namelist:
        basename = n.rstrip('/').split('/')[0]
        if is_dist_info_dir(basename):
            dist_info_dirs.add(basename)
    if len(dist_info_dirs) > 1:
        raise DistInfoError('Wheel contains multiple .dist-info directories')
    elif len(dist_info_dirs) == 1:
        dist_info_dir = next(iter(dist_info_dirs))
        diname, _, diversion = dist_info_dir[:-len(".dist-info")].partition('-')
        if (
            canonicalize_name(diname) != canon_project
            or canonicalize_version(diversion.replace('_', '-'))
                != canon_version
        ):
            raise DistInfoError(
                f"Project & version of wheel's .dist-info directory do not"
                f" match wheel name: {dist_info_dir!r}"
            )
        return dist_info_dir
    else:
        raise DistInfoError('No .dist-info directory in wheel')



import pytest
from   wheel_inspect.errors import DistInfoError
from   wheel_inspect.util   import extract_modules, find_dist_info_dir, \
                                    is_data_dir, is_dist_info_dir, \
                                    split_content_type, split_keywords

@pytest.mark.parametrize('kwstr,expected', [
    (
        'pypi,warehouse,search,packages,pip',
        (['pypi', 'warehouse', 'search', 'packages', 'pip'], ','),
    ),
    (
        'pypi warehouse search packages pip',
        (['pypi', 'warehouse', 'search', 'packages', 'pip'], ' '),
    ),
    (
        "pypi,pep503,simple repository api,packages,pip",
        (["pypi", "pep503", "simple repository api", "packages", "pip"], ','),
    ),
    ('', ([], ' ')),
    (' ', ([], ' ')),
    (',', ([], ',')),
    (' , ', ([], ',')),
    (' , , ', ([], ',')),
    ('foo', (['foo'], ' ')),
    ('foo,bar', (['foo', 'bar'], ',')),
    ('foo, bar', (['foo', 'bar'], ',')),
    ('foo ,bar', (['foo', 'bar'], ',')),
    (' foo , bar ', (['foo', 'bar'], ',')),
    (' foo , , bar ', (['foo', 'bar'], ',')),
    ('foo,,bar', (['foo', 'bar'], ',')),
    (',foo', (['foo'], ',')),
    ('foo,', (['foo'], ',')),
])
def test_split_keywords(kwstr, expected):
    assert split_keywords(kwstr) == expected

@pytest.mark.parametrize('filelist,modules', [
    (
        [
            "qypi/__init__.py",
            "qypi/__main__.py",
            "qypi/api.py",
            "qypi/util.py",
            "qypi-0.4.1.dist-info/DESCRIPTION.rst",
            "qypi-0.4.1.dist-info/LICENSE.txt",
            "qypi-0.4.1.dist-info/METADATA",
            "qypi-0.4.1.dist-info/RECORD",
            "qypi-0.4.1.dist-info/WHEEL",
            "qypi-0.4.1.dist-info/entry_points.txt",
            "qypi-0.4.1.dist-info/metadata.json",
            "qypi-0.4.1.dist-info/top_level.txt",
        ],
        ["qypi", "qypi.__main__", "qypi.api", "qypi.util"],
    ),

    (
        [
            "flit/__init__.py",
            "flit/__main__.py",
            "flit/_get_dirs.py",
            "flit/build.py",
            "flit/common.py",
            "flit/inifile.py",
            "flit/init.py",
            "flit/install.py",
            "flit/installfrom.py",
            "flit/log.py",
            "flit/logo.py",
            "flit/sdist.py",
            "flit/upload.py",
            "flit/wheel.py",
            "flit/license_templates/apache",
            "flit/license_templates/gpl3",
            "flit/license_templates/mit",
            "flit/vcs/__init__.py",
            "flit/vcs/git.py",
            "flit/vcs/hg.py",
            "flit/vendorized/__init__.py",
            "flit/vendorized/readme/__init__.py",
            "flit/vendorized/readme/clean.py",
            "flit/vendorized/readme/rst.py",
            "flit-0.11.1.dist-info/entry_points.txt",
            "flit-0.11.1.dist-info/LICENSE",
            "flit-0.11.1.dist-info/WHEEL",
            "flit-0.11.1.dist-info/METADATA",
            "flit-0.11.1.dist-info/RECORD",
        ],
        [
            "flit",
            "flit.__main__",
            "flit._get_dirs",
            "flit.build",
            "flit.common",
            "flit.inifile",
            "flit.init",
            "flit.install",
            "flit.installfrom",
            "flit.log",
            "flit.logo",
            "flit.sdist",
            "flit.upload",
            "flit.vcs",
            "flit.vcs.git",
            "flit.vcs.hg",
            "flit.vendorized",
            "flit.vendorized.readme",
            "flit.vendorized.readme.clean",
            "flit.vendorized.readme.rst",
            "flit.wheel",
        ],
    ),

    (
        [
            'cmarkgfm-0.4.2.dist-info/WHEEL',
            'cmarkgfm-0.4.2.dist-info/top_level.txt',
            'cmarkgfm-0.4.2.dist-info/METADATA',
            'cmarkgfm-0.4.2.dist-info/RECORD',
            'cmarkgfm/_cmark.abi3.so',
            'cmarkgfm/cmark_module.h',
            'cmarkgfm/__init__.py',
            'cmarkgfm/cmark.py',
            'cmarkgfm/build_cmark.py',
            'cmarkgfm/cmark.cffi.h',
        ],
        [
            'cmarkgfm',
            'cmarkgfm._cmark',
            'cmarkgfm.build_cmark',
            'cmarkgfm.cmark',
        ],
    ),

    (
        [
            'cmarkgfm-0.4.2.dist-info/WHEEL',
            'cmarkgfm-0.4.2.dist-info/top_level.txt',
            'cmarkgfm-0.4.2.dist-info/METADATA',
            'cmarkgfm-0.4.2.dist-info/RECORD',
            'cmarkgfm/_cmark.cp37-win_amd64.pyd',
            'cmarkgfm/cmark_module.h',
            'cmarkgfm/__init__.py',
            'cmarkgfm/cmark.py',
            'cmarkgfm/build_cmark.py',
            'cmarkgfm/cmark.cffi.h',
        ],
        [
            'cmarkgfm',
            'cmarkgfm._cmark',
            'cmarkgfm.build_cmark',
            'cmarkgfm.cmark',
        ],
    ),

    (
        [
            'cmarkgfm-0.4.2.dist-info/WHEEL',
            'cmarkgfm-0.4.2.dist-info/top_level.txt',
            'cmarkgfm-0.4.2.dist-info/METADATA',
            'cmarkgfm-0.4.2.dist-info/RECORD',
            'cmarkgfm/_cmark.so',
            'cmarkgfm/cmark_module.h',
            'cmarkgfm/__init__.py',
            'cmarkgfm/cmark.py',
            'cmarkgfm/build_cmark.py',
            'cmarkgfm/cmark.cffi.h',
        ],
        [
            'cmarkgfm',
            'cmarkgfm._cmark',
            'cmarkgfm.build_cmark',
            'cmarkgfm.cmark',
        ],
    ),

    (
        [
            'mxnet_coreml_converter-0.1.0a7.data/purelib/converter/__init__.py',
            'mxnet_coreml_converter-0.1.0a7.data/purelib/converter/_add_pooling.py',
            'mxnet_coreml_converter-0.1.0a7.data/purelib/converter/_layers.py',
            'mxnet_coreml_converter-0.1.0a7.data/purelib/converter/_mxnet_converter.py',
            'mxnet_coreml_converter-0.1.0a7.data/purelib/converter/utils.py',
            'mxnet_coreml_converter-0.1.0a7.data/scripts/mxnet_coreml_converter.py',
            'mxnet_coreml_converter-0.1.0a7.dist-info/DESCRIPTION.rst',
            'mxnet_coreml_converter-0.1.0a7.dist-info/metadata.json',
            'mxnet_coreml_converter-0.1.0a7.dist-info/top_level.txt',
            'mxnet_coreml_converter-0.1.0a7.dist-info/WHEEL',
            'mxnet_coreml_converter-0.1.0a7.dist-info/METADATA',
            'mxnet_coreml_converter-0.1.0a7.dist-info/RECORD',
        ],
        [
            'converter',
            'converter._add_pooling',
            'converter._layers',
            'converter._mxnet_converter',
            'converter.utils',
        ],
    ),

    (
        [
            '',
            'foo-1.0.data/platlib/foo/__init__.py',
            'foo-1.0.data/platlib/foo/def.py',
            'foo-1.0.data/platlib/foo/has-hyphen.py',
            'foo-1.0.data/platlib/foo/extra.ext.py',
            'foo-1.0.dist-info/METADATA',
            'foo-1.0.dist-info/WHEEL',
            'foo-1.0.dist-info/RECORD',
            'foo-1.0.dist-info/glarch.py',
        ],
        ['foo'],
    ),

    (
        [
            'Acquisition/Acquisition.h',
            'Acquisition/_Acquisition.c',
            'Acquisition/_Acquisition.pyd',
            'Acquisition/__init__.py',
            'Acquisition/interfaces.py',
            'Acquisition/tests.py',
            'Acquisition-4.6.dist-info/LICENSE.txt',
            'Acquisition-4.6.dist-info/METADATA',
            'Acquisition-4.6.dist-info/WHEEL',
            'Acquisition-4.6.dist-info/top_level.txt',
            'Acquisition-4.6.dist-info/RECORD',
        ],
        [
            'Acquisition',
            'Acquisition._Acquisition',
            'Acquisition.interfaces',
            'Acquisition.tests',
        ],
    ),

])
def test_extract_modules(filelist, modules):
    assert extract_modules(filelist) == modules

@pytest.mark.parametrize('s,ct', [
    ('text/plain', ('text', 'plain', {})),
    ('text/plain; charset=utf-8', ('text', 'plain', {"charset": "utf-8"})),
    (
        'text/markdown; charset=utf-8; variant=GFM',
        ('text', 'markdown', {"charset": "utf-8", "variant": "GFM"}),
    ),
])
def test_split_content_type(s, ct):
    assert split_content_type(s) == ct

@pytest.mark.parametrize('name,expected', [
    ('somepackage-1.0.0.dist-info', True),
    ('somepackage.dist-info', False),
    ('somepackage-1.0.0-1.dist-info', False),
    ('somepackage-1.0.0.data', False),
    ('SOME_._PaCkAgE-0.dist-info', True),
    ('foo-1!2+local.dist-info', True),
    ('foo-1_2_local.dist-info', True),
    ('.dist-info', False),
])
def test_is_dist_info_dir(name, expected):
    assert is_dist_info_dir(name) is expected

@pytest.mark.parametrize('name,expected', [
    ('somepackage-1.0.0.data', True),
    ('somepackage.data', False),
    ('somepackage-1.0.0-1.data', False),
    ('somepackage-1.0.0.dist-info', False),
    ('SOME_._PaCkAgE-0.data', True),
    ('foo-1!2+local.data', True),
    ('foo-1_2_local.data', True),
    ('.data', False),
])
def test_is_data_dir(name, expected):
    assert is_data_dir(name) is expected

@pytest.mark.parametrize('namelist,project,version,expected', [
    (
        [
            "foo.py",
            "foo-1.0.dist-info/WHEEL",
            "foo-1.0.dist-info/RECORD",
        ],
        "foo",
        "1.0",
        "foo-1.0.dist-info",
    ),
    (
        [
            "foo.py",
            "FOO-1.0.0.dist-info/WHEEL",
            "FOO-1.0.0.dist-info/RECORD",
        ],
        "foo",
        "1.0",
        "FOO-1.0.0.dist-info"
    ),
    (
        [
            "foo.py",
            "foo-1.dist-info/WHEEL",
            "foo-1.dist-info/RECORD",
        ],
        "foo",
        "1.0",
        "foo-1.dist-info"
    ),
    (
        [
            "foo.py",
            "FOO-1.0_1.dist-info/WHEEL",
            "FOO-1.0_1.dist-info/RECORD",
        ],
        "foo",
        "1.0.post1",
        "FOO-1.0_1.dist-info",
    ),
])
def test_find_dist_info_dir(namelist, project, version, expected):
    assert find_dist_info_dir(namelist, project, version) == expected

@pytest.mark.parametrize('namelist,project,version,msg', [
    (
        [
            "foo.py",
            "foo-1.0.dist/WHEEL",
        ],
        "foo",
        "1.0",
        'No .dist-info directory in wheel',
    ),
    (
        [
            "foo.py",
            "bar-1.0.dist-info/WHEEL",
        ],
        "foo",
        "1.0",
        "Project & version of wheel's .dist-info directory do not match wheel"
        " name: 'bar-1.0.dist-info'"
    ),
    (
        [
            "foo.py",
            "foo-2.0.dist-info/WHEEL",
        ],
        "foo",
        "1.0",
        "Project & version of wheel's .dist-info directory do not match wheel"
        " name: 'foo-2.0.dist-info'"
    ),
    (
        [
            "foo.py",
            "foo-1.0.dist-info/WHEEL",
            "bar-2.0.dist-info/RECORD",
        ],
        "foo",
        "1.0",
        'Wheel contains multiple .dist-info directories',
    ),
    (
        [
            "foo.py",
            "FOO-1.0.0.dist-info/WHEEL",
            "foo-1.dist-info/RECORD",
        ],
        "foo",
        "1.0",
        'Wheel contains multiple .dist-info directories',
    ),
    (
        ["foo.py", ".dist-info/WHEEL"],
        "foo",
        "1.0",
        'No .dist-info directory in wheel',
    ),
])
def test_find_dist_info_dir_error(namelist, project, version, msg):
    with pytest.raises(DistInfoError) as excinfo:
        find_dist_info_dir(namelist, project, version)
    assert str(excinfo.value) == msg

### TODO: Add more test cases for all functions!
